#
class ArtichokeUser(object):
    def __init__(self, user_name, password):
        self.user_name = user_name
        self.password = password

class CookieAuthenticator(object):
    def __init__(self, cookie_name='artichoke_auth'):
        self.cookie_name = cookie_name

    def authenticate(self, request):
        request.identity = None

        auth_cookie = request.cookies.get(self.cookie_name)
        if auth_cookie:
            cookie_value = auth_cookie.decode('base64')
            login, password = cookie_value.split(':', 1)
            request.identity = {'user': ArtichokeUser(login, password)}

    def inject_cookie(self, response):
        skip_user = False

        try:
            artichoke_user = response.identity and response.identity['user']
        except:
            skip_user = True

        if not skip_user:
            if artichoke_user:
                cookie_value = u'%s:%s' % (artichoke_user.user_name, artichoke_user.password)
                cookie_value = cookie_value.encode('utf-8')
                cookie_value = cookie_value.encode('base64').rstrip()

                response.set_cookie(self.cookie_name, cookie_value, path='/')
            else:
                response.delete_cookie(self.cookie_name)






import artichoke, os, mimetypes, base64
from artichoke import request, response, expose, url, redirect, flash
from artichoke import FormBuilder

class SubController(artichoke.Controller):
    @expose()
    def about(self, args, params):
        return 'HELLO %s' % str(args)

    @expose()
    def index(self, args, params):
        return 'INDEX'

    @expose()
    def not_found(self, args, params):
        return 'Sub Not Found'

class RootController(artichoke.Controller):
    def __init__(self, application, templates_path, helpers):
        super(RootController, self).__init__(application, templates_path, helpers)
        self.sub = SubController(application, os.path.join(templates_path, 'sub'), helpers)

    @expose('index')
    def index(self, args, params):
        return dict(foo=False, form=FormBuilder('/echo_image', {'image':{'type':'file'}}))

    @expose()
    def echo_image(self, args, params):
        if params.has_key('image'):
            image = params['image']
            value = "data:%s;base64," % (mimetypes.guess_type(image.filename)[0] or "application/octet-stream")
            value += base64.b64encode(image.file.read())
        else:
            value = ''

        s = '''<html>
<head>
</head>
<body>
    <img src="%s"/>
</body>
</html>''' % value
        return s

    @expose('index')
    def other(self, args, params):
        foo = params.get('foo', False)
        flash('Flash here!')
        return dict(foo=foo)

    @expose()
    def crash(self, args, params):
        raise Exception('FATAL')

    def hidden(self, args, params):
        return 'HIDDEN'

    @expose()
    def login(self, args, params):
        class FakeUser(object):
            pass

        user = FakeUser()
        user.user_name = params['user']
        user.password = params['password']

        response.identity = {'user':user}
        flash('Welcome back!')
        return redirect('/index')

    @expose()
    def logout(self, args, params):
        response.identity = None
        return redirect('/index')


app = artichoke.Application(root=RootController, templates_path='views')

if __name__ == '__main__':
    from artichoke.server import serve
    serve(app)



import sys, cgitb
import email, mimetools, cStringIO
import smtplib

__all__ = ['create_mail', 'send_mail', 'send_traceback']

def create_mail(sender, recipient, subject, html, text):
    out = cStringIO.StringIO()
    htmlin = cStringIO.StringIO(html)
    txtin = cStringIO.StringIO(text)

    writer = MimeWriter.MimeWriter(out)
    writer.addheader("From", sender)
    writer.addheader("To", recipient)
    writer.addheader("Subject", subject)
    writer.addheader("MIME-Version", "1.0")
    writer.startmultipartbody("alternative")
    writer.flushheaders()

    subpart = writer.nextpart()
    subpart.addheader("Content-Transfer-Encoding", "quoted-printable")
    pout = subpart.startbody("text/plain", [("charset", 'us-ascii')])
    mimetools.encode(txtin, pout, 'quoted-printable')
    txtin.close()

    subpart = writer.nextpart()
    subpart.addheader("Content-Transfer-Encoding", "quoted-printable")

    pout = subpart.startbody("text/html", [("charset", 'us-ascii')])
    mimetools.encode(htmlin, pout, 'quoted-printable')
    htmlin.close()
    writer.lastpart()
    msg = out.getvalue()
    out.close()
    return msg

def send_mail(sender, recipient, subject, html, text):
    message = create_mail(sender, recipient, subject, html, text)
    server = smtplib.SMTP("localhost")
    server.sendmail(sender, recipient, message)
    server.quit()

def format_traceback():
    return cgitb.html(sys.exc_info())

def send_traceback(mail_from, mail_to):
    try:
        send_mail(mail_from, mail_to, 'Artichoke Exception',
                  format_traceback(), cgitb.text(sys.exc_info()))
    except:
        print 'Failed to connect to SMTP'



import genshi

class FormBuilder(object):
    def __init__(self, action, fields, fields_order=None):
        self.fields = fields
        self.action = action

        if not fields_order:
            self.fields_order = self.fields.keys()
        else:
            self.fields_order = fields_order

    def render(self):
        form = ''
        form += '<form action="%(action)s" method="post" enctype="multipart/form-data">\n' % dict(action=self.action)
        form += '<table>\n'

        for field_name in self.fields_order:
            field_attrs = self.fields[field_name]

            form += '<tr>\n'
            form += '<td id="%s_label">%s</td>\n' % (field_name, field_attrs.get('label', '%s:' % field_name.capitalize()))
            form += '<td id="%s_field">' % field_name
            field_type = field_attrs.get('type', 'text')
            if field_type == 'textarea':
                form += '<textarea name="%s"></textarea>' % field_name
            else:
                form += '<input type="%s" name="%s"/>' % (field_type, field_name)
            form += '</td>\n'
            form += '</tr>\n'

        form += '''<tr>
                    <td id="submit_label"></td>
                    <td id="submit_field"><input type="submit" value="Submit"/></td>
                </tr>\n'''
        form += '</table>\n'
        form += '</form>\n'
        return genshi.Markup(form)



from webob import Request, Response
from webob.exc import HTTPFound
from authenticator import CookieAuthenticator
from paste.registry import RegistryManager
from mail_traceback import send_traceback, format_traceback
import traceback, mimetypes, os
from controller import request as a_request, response as a_response

class ArtichokeHelpers(object):
    pass

class ArtichokeCore(object):
    def __init__(self, root, templates_path, config={}):
        self.root = root(self, templates_path, config.get('helpers', ArtichokeHelpers()))
        self.statics = config.get('statics', 'public')
        self.autoreload_templates = config.get('autoreload', False)

        requested_authenticator = config.get('authenticator')
        if not requested_authenticator:
            self.authenticator = CookieAuthenticator()
        else:
            self.authenticator = requested_authenticator()

        self.mail_errors_to = config.get('mail_errors_to')
        self.mail_errors_from = config.get('mail_errors_from', 'artichoke@localhost')
        self.traceback = config.get('traceback', True)
        self.force_request_encoding = config.get('force_request_encoding', 'utf-8')

    def __call__(self, environ, start_response):
        request = Request(environ=environ)
        environ['paste.registry'].register(a_request, request)

        response = Response(body="Artichoke Default Page", content_type='text/html', charset='utf-8')
        environ['paste.registry'].register(a_response, response)

        if self.force_request_encoding:
            request.charset = self.force_request_encoding

        static_path = self.statics + request.path_info
        if os.path.exists(static_path) and os.path.isfile(static_path):
            response.body = open(static_path).read()
            response.status = 200
            response.content_type = mimetypes.guess_type(static_path)[0]
        else:
            try:
                self.authenticator.authenticate(request)
                self.root._dispatch(request)
                self.authenticator.inject_cookie(response)
            except HTTPFound, e:
                response = e
                self.authenticator.inject_cookie(response)
            except Exception, e:
                if self.mail_errors_to:
                    send_traceback(self.mail_errors_from, self.mail_errors_to)

                if self.traceback:
                    format_error_body = traceback.format_exc()
                    try:
                        format_error_body = format_traceback()
                        format_content_type = 'text/html'
                    except:
                        format_content_type = 'text/plain'
                    response.body = format_error_body
                    response.status = 500
                    response.content_type = format_content_type
                else:
                    response.body = "Internal Server Error"
                    response.status = 500
        return response(environ, start_response)

class Application(object):
    def __init__(self, root, templates_path, config={}):
        self.inner_app = ArtichokeCore(root, templates_path, config)
        self.registry_manager = RegistryManager(self.inner_app)

    def __call__(self, environ, start_response):
        return self.registry_manager(environ, start_response)



from webob import Request, Response
from webob.exc import HTTPFound
from genshi.template import TemplateLoader
import inspect, pickle, base64, types, urllib
from paste.registry import StackedObjectProxy

request = StackedObjectProxy(name="request")
response = StackedObjectProxy(name="response")

def expose(template = None,
           content_type = 'text/html'):

    if template == 'json':
        content_type = 'application/json'

    def decorate(f):
        f.exposed = True

        if not hasattr(f, 'artichoke'):
            f.artichoke = {}

        f.artichoke['template'] = template
        f.artichoke['content-type'] = content_type
        return f
    return decorate

def url(base_url, params={}):
    if base_url.startswith('/'):
        base_url = request.environ['SCRIPT_NAME'] + base_url

    if params:
        sane_params = {}
        for key in params.iterkeys():
            sane_params[key] = params[key].encode('utf-8')
        return '?'.join((base_url, urllib.urlencode(sane_params)))

    return base_url


def redirect(where):
    exc = HTTPFound(location=where)
    try:
        exc.identity = response.identity
    except:
        pass

    if response.flash_obj:
        decoded_flash = pickle.dumps(response.flash_obj)
        exc.set_cookie('flash_obj', base64.b64encode(decoded_flash))

    raise exc

def flash(msg, css_class='warning'):
    response.flash_obj = {'msg':msg, 'class':css_class}

class Controller(object):
    def __init__(self, application, template_path, helpers):
        self.template_path = template_path
        self.loader = TemplateLoader([template_path])
        self.helpers = helpers
        self.templates = {}
        self.application = application

    def render(self, template, params):
        if not self.application.autoreload_templates:
            try:
                tmpl = self.templates[template]
            except:
                tmpl = self.loader.load(template)
                self.templates[template] = tmpl
        else:
            self.loader = TemplateLoader([self.template_path])
            tmpl = self.loader.load(template)

        stream = tmpl.generate(**params)
        return stream.render('xhtml')

    @expose()
    def not_found(self, args, params):
        response.content_type = 'text/html'
        response.status = 404
        return '''
<html>
   <head>
       <title>Page not Found</title>
   </head>
   <body>
       <h1>Page Not Found</h1>
       <div style="font-size:small">Artichoke Framework</div>
   </body>
</html>
'''

    def _dispatch(self, request):
        path = request.path_info.split('/')
        while path and not path[0]:
            path.pop(0)

        if not path:
            path = ['index']

        members = dict(inspect.getmembers(self))
        call = self

        if isinstance(call, Controller):
            while path[0:]:
                subpath = path[0]
                members = dict(inspect.getmembers(call))
                try:
                    call_candidate = members[subpath]
                    if isinstance(call_candidate, Controller) or \
                       (hasattr(call_candidate, 'exposed') and call_candidate.exposed):
                        call = call_candidate
                        path = path[1:]
                    else:
                        break
                except:
                    break

        if isinstance(call, Controller):
            if path:
                call = call.not_found
            else:
                call = call.index

        self.do_call(call, request, path[:])

    def do_call(self, call, request, path):
        response.content_type = call.artichoke['content-type']
        self.inject_tools(request, response)

        if call.artichoke['template']:
            tmpl_context = {}
            tmpl_context['a'] = type('Bunch', (object,), {'url':staticmethod(url)})
            tmpl_context['h'] = self.helpers
            tmpl_context['request'] = request
            tmpl_context['response'] = response
            tmpl_context.update(call(path, request.params))

            template = call.artichoke['template'] + '.html'
            response.body = self.render(template, tmpl_context)
        else:
            response.body = call(path, request.params)

        return response

    def inject_tools(self, request, response):
        response.flash_obj = {}
        if request.cookies.get('flash_obj'):
            try:
                decoded_flash = request.cookies['flash_obj'].decode('base64')
                response.flash_obj = pickle.loads(decoded_flash)
            except Exception, e:
                print e
            response.delete_cookie('flash_obj')



from setuptools import setup, find_packages
import sys, os

version = '0.3'

setup(name='artichoke',
      version=version,
      description="Simple and Lightweight WSGI Web Framework",
      long_description="""\
Artichoke is a lightweight WSGI Python framework for rapid prototyping of web applications. Artichoke has been developed with a syntax similar to the one of the Turbogears2 framework to permit to develop fast and small web applications which can be quickly switched to a full stack framework when necessary.
""",
      classifiers=["Development Status :: 5 - Production/Stable",
                   "License :: OSI Approved :: GNU General Public License (GPL)",
                   "Topic :: Internet :: WWW/HTTP :: WSGI :: Middleware",
                   "Programming Language :: Python",
                   "Topic :: Internet :: WWW/HTTP :: WSGI",
                   "Topic :: Internet :: WWW/HTTP :: WSGI :: Application",
                   "Topic :: Software Development :: Libraries :: Application Frameworks",
                   "Environment :: Web Environment"],
      keywords='wsgi web simple light',
      author='AXANT',
      author_email='tech@axant.it',
      url='http://www.axantlabs.com/artichoke',
      license='GPL',
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'genshi',
          'webob',
          "paste"
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )
